#!/bin/sh

###############################################################################
##                                                                           ##
##    Copyright (c) 2010 by Steffen Beyer.                                   ##
##    All rights reserved.                                                   ##
##                                                                           ##
##    This program is free software; you can redistribute it                 ##
##    and/or modify it under the same terms as Perl itself.                  ##
##                                                                           ##
###############################################################################

umask 022

FORM="%Y_%m_%d_%H_%M_%S"
TIME=`/bin/date "+%s"`
TIME=`/usr/bin/expr "$TIME" - 2592000`
DATE=`/bin/date "+$FORM"`
FILE="data.bin"
ROOT="/home4/fluomedi/public_html/sb/volley"
PATH="backups"

SOURCE="${ROOT}/${FILE}"
BACKUP="${ROOT}/${PATH}"
TARGET="${BACKUP}/${DATE}_${FILE}"

if [ ! -f "$SOURCE" ]
then
    echo "source file <$SOURCE> does not exist!" >&2
    exit 1
fi

if [ ! -s "$SOURCE" ]
then
    echo "source file <$SOURCE> is empty!" >&2
    exit 1
fi

if [ -e "$TARGET" ]
then
    echo "target file <$TARGET> already exists!" >&2
    exit 1
fi

# Don't use the copy option "-p" here if "stat" is used further below,
# or the purging loop at the bottom will delete more than it should!!!
# But when the timestamp is retrieved with "date" instead, use "-p"!

/bin/cp -p "$SOURCE" "$TARGET"
if [ $? -ne 0 ]
then
    echo "can't copy file <$SOURCE> to <$TARGET>!" >&2
    exit 1
fi

if [ ! -f "$TARGET" ]
then
    echo "target file <$TARGET> was not copied!" >&2
    exit 1
fi

if [ ! -s "$TARGET" ]
then
    echo "target file <$TARGET> is empty!" >&2
    exit 1
fi

/bin/chmod 444 "$TARGET"
if [ $? -ne 0 ]
then
    echo "can't chmod file <$TARGET>!" >&2
fi
 
for ITEM in `/bin/ls -1 "$BACKUP"`
do
    FILE="$BACKUP/$ITEM"
    STAT=`/bin/date -r "$FILE" "+%s" 2>/dev/null`
    if [ "X$STAT" = "X" ]
    then
        echo "can't stat file <$FILE>!" >&2
    else
        if [ "$STAT" -eq 0 ]
        then
            echo "zero time for file <$FILE>!" >&2
        else
            if [ "$STAT" -lt "$TIME" ]
            then
                /bin/rm -f "$FILE"
                if [ $? -ne 0 ]
                then
                    echo "can't remove file <$FILE>!" >&2
                fi
            fi
        fi
    fi
done

