#!/usr/local/bin/perl -w

###############################################################################
##                                                                           ##
##    Copyright (c) 2010 by Steffen Beyer.                                   ##
##    All rights reserved.                                                   ##
##                                                                           ##
##    This program is free software; you can redistribute it                 ##
##    and/or modify it under the same terms as Perl itself.                  ##
##                                                                           ##
###############################################################################

use strict;

use Date::Calc qw(:all);
use Storable qw(lock_nstore lock_retrieve);

umask(022);

my $Root    = '../';
my $Path    = 'backups/';
my $Const   = 'const.bin';
my $File    = 'data.bin';
my $Data    = {};
my $Fixed   = {};
my $Replace = {};
my $Lang    = 1;
my @Today   = Today_and_Now();
my $Today   = Date_to_Days(@Today[0,1,2]);
my $TIME    = sprintf("%04d_%02d_%02d_%02d_%02d_%02d", @Today);
my $HTML    = [];
my $BOLD    = '<FONT COLOR="#FF0000"><B>';
my $NORM    = '</B></FONT>';

my $Orig_Const = $Root . $Const;
my $Orig_Data  = $Root . $File;
my $Bkup_Const = $Root . $Path . '_' . $TIME . '_' . $Const;
my $Bkup_Data  = $Root . $Path . '_' . $TIME . '_' . $File;

my($key,$txt,$dow,$fat,$end);

eval
{
    init_tables();
    process_query_string();
    $Fixed = lock_retrieve($Orig_Const);
    $key = keys(%$Fixed);
    if ($key == 1) { $txt = "There is currently one fixed event:";      }
    else           { $txt = "There are currently ${key} fixed events:"; }
    execute_command();
};
if ($@)
{
    $txt = $@;
    $txt =~ s/\sat\s.+\sline\s\d+.*$//;
    $txt = ucfirst($txt);
    $txt = "Fatal error: $txt";
}

print <<"VERBATIM";
Content-type: text/html; charset="iso-8859-1"

<HTML>
<HEAD>
    <META HTTP-EQUIV="Content-Type" CONTENT="text/html; charset=iso-8859-1">
    <TITLE>EPO Volleyball Club Fixed Event Administration</TITLE>
</HEAD>
<BODY>
<CENTER>
    <P>
    <HR NOSHADE SIZE="2">
    <P>
    <H1>EPO Volleyball Club Fixed Event Administration</H1>
    <P>
    <HR NOSHADE SIZE="2">
    <P>
    <H3>${txt}</H3>
    <P>
    <TABLE BGCOLOR="#F4F4F4" CELLSPACING="1" CELLPADDING="15" BORDER="2">
    <FORM METHOD="GET" ACTION="">
VERBATIM

foreach $key (1..7)
{
    $dow = Day_of_Week_to_Text($key,$Lang);
    if (exists $Fixed->{$key} and defined $Fixed->{$key} and $Fixed->{$key} ne '')
    {
        $txt = $Fixed->{$key};
        $fat = $BOLD;
        $end = $NORM;
    }
    else
    {
        $txt = $fat = $end = '';
    }
    print <<"VERBATIM";
    <TR>
    <TD ALIGN="RIGHT"  VALIGN="middle">${fat}${key}${end}</TD>
    <TD ALIGN="LEFT"   VALIGN="middle">${fat}${dow}${end}</TD>
    <TD ALIGN="CENTER" VALIGN="middle"><TEXTAREA NAME="Day${key}" ROWS="6" COLS="80" WRAP="physical">${txt}</TEXTAREA></TD>
    </TR>
VERBATIM
}

print <<"VERBATIM";
    <TR>
    <TD COLSPAN="3" ALIGN="center" VALIGN="middle"><INPUT TYPE="reset" VALUE="Undo all changes"></TD>
    </TR>
    <TR>
    <TD COLSPAN="3" ALIGN="center" VALIGN="middle"><INPUT TYPE="submit" VALUE="Save all changes"></TD>
    </TR>
    </FORM>
    </TABLE>
    <P>
    <HR NOSHADE SIZE="2">
    <P>
    <H1>EPO Volleyball Club Fixed Event Administration</H1>
    <P>
    <HR NOSHADE SIZE="2">
    <P>
</CENTER>
</BODY>
</HTML>
VERBATIM

sub process_query_string
{
    my $query = $ENV{'QUERY_STRING'} || $ENV{'REDIRECT_QUERY_STRING'} || join('&', @ARGV) || '';
    my @pairs = split(/&/, $query);
    my($pair,$var,$val,$key);

    foreach $pair (@pairs)
    {
        ($var,$val) = split(/=/,$pair,2);
        if ($var =~ /^Day([1-7])$/i)
        {
            $key = $1;
            $val =~ s/\+/ /g;
            $val =~ s/%([0-9A-Fa-f][0-9A-Fa-f])/chr(hex($1))/eg;
            $val =~ s/^\s+//gm;
            $val =~ s/\s+$//gm;
            $val =~ s/[\t ][\t ]+/ /g;
            $val =~ s/ ([,.:;!?])/$1/ge;
            $val = html($val);
            $Replace->{$key} = $val if $val ne '';
        }
    }
}

sub execute_command
{
    my($run,$key,$old,$new,$dow,$n);
    my $count = keys(%{$Replace});
    my $save = 0;
    if ($count > 0)
    {
        $run = 0;
        KEY:
        foreach $key (1..7)
        {
            $old = (exists $Fixed->{$key}   and defined $Fixed->{$key}   and $Fixed->{$key}   ne '') ? 1 : 0;
            $new = (exists $Replace->{$key} and defined $Replace->{$key} and $Replace->{$key} ne '') ? 1 : 0;
            if
            (
                ($old and $new and $Fixed->{$key} ne $Replace->{$key}) or ($old and not $new)
            )
            {
                $run = 1;
                last KEY;
            }
        }
        if ($run)
        {
            if ($count == 1) { $txt = "Saved one fixed event:";       }
            else             { $txt = "Saved ${count} fixed events:"; }
            $Data = lock_retrieve( $Orig_Data );
            lock_nstore( $Fixed, $Bkup_Const );
            lock_nstore( $Data,  $Bkup_Data  );
#           foreach $key (keys %{$Data})
#           {
#               $dow = $key - int(($key-1)/7) * 7;
#               if ($key >= $Today and exists $Fixed->{$dow} and defined $Fixed->{$dow} and $Fixed->{$dow} ne '')
#               {
#                   if (exists $Data->{$key} and defined $Data->{$key} and ref($Data->{$key}) eq 'ARRAY' and @{$Data->{$key}})
#                   {
#                       $n = 0;
#                       while ( $n < @{$Data->{$key}} )
#                       {
#                           if (exists $Replace->{$dow} and defined $Replace->{$dow} and $Replace->{$dow} ne '')
#                           {
#                               if ($Data->{$key}[$n][0] eq $Fixed->{$dow} and $Fixed->{$dow} ne $Replace->{$dow})
#                               {
#                                   $Data->{$key}[$n][0] = $Replace->{$dow};
#                                   $save = 1;
#                               }
#                               $n++;
#                           }
#                           else
#                           {
#                               if ($Data->{$key}[$n][0] eq $Fixed->{$dow})
#                               {
#                                   splice(@{$Data->{$key}},$n,1);
#                                   $save = 1;
#                               }
#                               else { $n++; }
#                           }
#                       }
#                   }
#               }
#           }
#           lock_nstore( $Data,    $Orig_Data  ) if ($save);
#           lock_nstore( $Replace, $Orig_Const );
        }
    }
}

sub html
{
    my($string) = $_[0];
    my($o);
    $string =~ s/(.)/(defined $HTML->[$o=ord($1)])?($HTML->[$o]||"&\#$o;"):$1/eg;
    return $string;
}

sub init_tables
{
    $HTML->[0x00] = '';
    $HTML->[0x01] = '';
    $HTML->[0x02] = '';
    $HTML->[0x03] = '';
    $HTML->[0x04] = '';
    $HTML->[0x05] = '';
    $HTML->[0x06] = '';
    $HTML->[0x07] = '';
    $HTML->[0x08] = '';
    # $HTML->[0x09] = '';
    # $HTML->[0x0A] = '';
    $HTML->[0x0B] = '';
    $HTML->[0x0C] = '';
    # $HTML->[0x0D] = '';
    $HTML->[0x0E] = '';
    $HTML->[0x0F] = '';
    $HTML->[0x10] = '';
    $HTML->[0x11] = '';
    $HTML->[0x12] = '';
    $HTML->[0x13] = '';
    $HTML->[0x14] = '';
    $HTML->[0x15] = '';
    $HTML->[0x16] = '';
    $HTML->[0x17] = '';
    $HTML->[0x18] = '';
    $HTML->[0x19] = '';
    $HTML->[0x1A] = '';
    $HTML->[0x1B] = '';
    $HTML->[0x1C] = '';
    $HTML->[0x1D] = '';
    $HTML->[0x1E] = '';
    $HTML->[0x1F] = '';
    $HTML->[0x22] = '&quot;';
    $HTML->[0x26] = '&amp;';
    # $HTML->[0x27] = '&apos;';
    $HTML->[0x3C] = '&lt;';
    $HTML->[0x3E] = '&gt;';
    $HTML->[0x7F] = '';
    $HTML->[0x80] = '';
    $HTML->[0x81] = '';
    $HTML->[0x82] = '';
    $HTML->[0x83] = '';
    $HTML->[0x84] = '';
    $HTML->[0x85] = '';
    $HTML->[0x86] = '';
    $HTML->[0x87] = '';
    $HTML->[0x88] = '';
    $HTML->[0x89] = '';
    $HTML->[0x8A] = '';
    $HTML->[0x8B] = '';
    $HTML->[0x8C] = '';
    $HTML->[0x8D] = '';
    $HTML->[0x8E] = '';
    $HTML->[0x8F] = '';
    $HTML->[0x90] = '';
    $HTML->[0x91] = '';
    $HTML->[0x92] = '';
    $HTML->[0x93] = '';
    $HTML->[0x94] = '';
    $HTML->[0x95] = '';
    $HTML->[0x96] = '';
    $HTML->[0x97] = '';
    $HTML->[0x98] = '';
    $HTML->[0x99] = '';
    $HTML->[0x9A] = '';
    $HTML->[0x9B] = '';
    $HTML->[0x9C] = '';
    $HTML->[0x9D] = '';
    $HTML->[0x9E] = '';
    $HTML->[0x9F] = '';
    $HTML->[0xA0] = '&nbsp;';
    $HTML->[0xA1] = '&iexcl;';
    $HTML->[0xA2] = '&cent;';
    $HTML->[0xA3] = '&pound;';
    $HTML->[0xA4] = '&curren;';
    $HTML->[0xA5] = '&yen;';
    $HTML->[0xA6] = '&brvbar;';
    $HTML->[0xA7] = '&sect;';
    $HTML->[0xA8] = '&uml;';
    $HTML->[0xA9] = '&copy;';
    $HTML->[0xAA] = '&ordf;';
    $HTML->[0xAB] = '&laquo;';
    $HTML->[0xAC] = '&not;';
    $HTML->[0xAD] = '&shy;';
    $HTML->[0xAE] = '&reg;';
    $HTML->[0xAF] = '&macr;';
    $HTML->[0xB0] = '&deg;';
    $HTML->[0xB1] = '&plusmn;';
    $HTML->[0xB2] = '&sup2;';
    $HTML->[0xB3] = '&sup3;';
    $HTML->[0xB4] = '&acute;';
    $HTML->[0xB5] = '&micro;';
    $HTML->[0xB6] = '&para;';
    $HTML->[0xB7] = '&middot;';
    $HTML->[0xB8] = '&cedil;';
    $HTML->[0xB9] = '&sup1;';
    $HTML->[0xBA] = '&ordm;';
    $HTML->[0xBB] = '&raquo;';
    $HTML->[0xBC] = '&frac14;';
    $HTML->[0xBD] = '&frac12;';
    $HTML->[0xBE] = '&frac34;';
    $HTML->[0xBF] = '&iquest;';
    $HTML->[0xC0] = '&Agrave;';
    $HTML->[0xC1] = '&Aacute;';
    $HTML->[0xC2] = '&Acirc;';
    $HTML->[0xC3] = '&Atilde;';
    $HTML->[0xC4] = '&Auml;';
    $HTML->[0xC5] = '&Aring;';
    $HTML->[0xC6] = '&AElig;';
    $HTML->[0xC7] = '&Ccedil;';
    $HTML->[0xC8] = '&Egrave;';
    $HTML->[0xC9] = '&Eacute;';
    $HTML->[0xCA] = '&Ecirc;';
    $HTML->[0xCB] = '&Euml;';
    $HTML->[0xCC] = '&Igrave;';
    $HTML->[0xCD] = '&Iacute;';
    $HTML->[0xCE] = '&Icirc;';
    $HTML->[0xCF] = '&Iuml;';
    $HTML->[0xD0] = '&ETH;';
    $HTML->[0xD1] = '&Ntilde;';
    $HTML->[0xD2] = '&Ograve;';
    $HTML->[0xD3] = '&Oacute;';
    $HTML->[0xD4] = '&Ocirc;';
    $HTML->[0xD5] = '&Otilde;';
    $HTML->[0xD6] = '&Ouml;';
    $HTML->[0xD7] = '&times;';
    $HTML->[0xD8] = '&Oslash;';
    $HTML->[0xD9] = '&Ugrave;';
    $HTML->[0xDA] = '&Uacute;';
    $HTML->[0xDB] = '&Ucirc;';
    $HTML->[0xDC] = '&Uuml;';
    $HTML->[0xDD] = '&Yacute;';
    $HTML->[0xDE] = '&THORN;';
    $HTML->[0xDF] = '&szlig;';
    $HTML->[0xE0] = '&agrave;';
    $HTML->[0xE1] = '&aacute;';
    $HTML->[0xE2] = '&acirc;';
    $HTML->[0xE3] = '&atilde;';
    $HTML->[0xE4] = '&auml;';
    $HTML->[0xE5] = '&aring;';
    $HTML->[0xE6] = '&aelig;';
    $HTML->[0xE7] = '&ccedil;';
    $HTML->[0xE8] = '&egrave;';
    $HTML->[0xE9] = '&eacute;';
    $HTML->[0xEA] = '&ecirc;';
    $HTML->[0xEB] = '&euml;';
    $HTML->[0xEC] = '&igrave;';
    $HTML->[0xED] = '&iacute;';
    $HTML->[0xEE] = '&icirc;';
    $HTML->[0xEF] = '&iuml;';
    $HTML->[0xF0] = '&eth;';
    $HTML->[0xF1] = '&ntilde;';
    $HTML->[0xF2] = '&ograve;';
    $HTML->[0xF3] = '&oacute;';
    $HTML->[0xF4] = '&ocirc;';
    $HTML->[0xF5] = '&otilde;';
    $HTML->[0xF6] = '&ouml;';
    $HTML->[0xF7] = '&divide;';
    $HTML->[0xF8] = '&oslash;';
    $HTML->[0xF9] = '&ugrave;';
    $HTML->[0xFA] = '&uacute;';
    $HTML->[0xFB] = '&ucirc;';
    $HTML->[0xFC] = '&uuml;';
    $HTML->[0xFD] = '&yacute;';
    $HTML->[0xFE] = '&thorn;';
    $HTML->[0xFF] = '&yuml;';
}

__END__

