#!/usr/local/bin/perl

###############################################################################
##                                                                           ##
##    Copyright (c) 2006 - 2010 by Steffen Beyer.                            ##
##    All rights reserved.                                                   ##
##                                                                           ##
##    This program is free software; you can redistribute it                 ##
##    and/or modify it under the same terms as Perl itself.                  ##
##                                                                           ##
###############################################################################

use strict;

use Date::Calc qw(:all);
use Storable qw(lock_retrieve);

my($Title) = 'EPO Volleyball Club Event Calendar Summary';
my($Const) = '../const.bin';
my($File)  = '../data.bin';
my($Data)  = {};
my($Fixed) = {};
my($Lang)  = 1;
my $Today  = Date_to_Days(Today());

if (-e $Const)
{
    eval { $Fixed = lock_retrieve($Const); };
    if ($@) { $Fixed = {}; }
}

if (-e $File)
{
    eval { $Data = lock_retrieve($File); };
    if ($@) { $Data = {}; }
}

my($dow,$key,$n,$m,$list,$item,$date,$name,$flag,@date);

my $List = [];
foreach $key (sort {$a <=> $b} keys %{$Data})
{
    if ($key >= $Today)
    {
        if (exists $Data->{$key} and defined $Data->{$key} and ref($Data->{$key}) eq 'ARRAY' and @{$Data->{$key}})
        {
            $list = [];
            $dow = $key - int(($key-1)/7) * 7;
            for ( $n = 0; $n < @{$Data->{$key}}; $n++ )
            {
                unless ($Data->{$key}[$n][0] eq $Fixed->{$dow})
                {
                    push(@{$list}, $n);
                }
            }
            push(@{$List}, [$key,$list]) if (@{$list});
        }
    }
}

process_query_string();

print <<"VERBATIM";
Content-type: text/html; charset="iso-8859-1"

<HTML>
<HEAD>
    <META HTTP-EQUIV="Content-Type" CONTENT="text/html; charset=iso-8859-1">
    <TITLE>$Title</TITLE>
</HEAD>
<BODY>
<CENTER>
    <P>
    <HR NOSHADE SIZE="2">
    <P>
    <H1>$Title</H1>
    <P>
    <HR NOSHADE SIZE="2">
    <P>
    <TABLE WIDTH="70%" BORDER="0"><TR><TD>
    <CENTER>
VERBATIM

if (@{$List})
{
    print qq{<TABLE BGCOLOR="#F4F4F4" CELLSPACING="1" CELLPADDING="15" BORDER="2">\n};
    foreach $item (@{$List})
    {
        $key  = $item->[0];
        $list = $item->[1];
        @date = Add_Delta_Days(1,1,1,$key-1);
        $date = Date_to_Text_Long(@date,$Lang);
        if (@{$list})
        {
            $flag = 0;
            foreach $n (@{$list})
            {
                print qq{    <TR>\n};
                $m = $n + 1;
                $name = $Data->{$key}[$n][0];
                if ($flag)
                {
                    print qq{        <TD ALIGN="left" VALIGN="middle">&nbsp;</TD>\n};
                    print qq{        <TD ALIGN="left" VALIGN="middle"><A HREF="/volley/?y=$date[0]&m=$date[1]&d=$date[2]&l=${Lang}#$m">#$m</A></TD>\n};
                    print qq{        <TD ALIGN="left" VALIGN="middle"><TEXTAREA ROWS="6" COLS="80" WRAP="physical">$name</TEXTAREA></TD>\n};
                }
                else
                {
                    print qq{        <TD ALIGN="left" VALIGN="middle"><A HREF="/volley/?y=$date[0]&m=$date[1]&d=$date[2]&l=${Lang}">$date</A></TD>\n};
                    print qq{        <TD ALIGN="left" VALIGN="middle"><A HREF="/volley/?y=$date[0]&m=$date[1]&d=$date[2]&l=${Lang}#$m">#$m</A></TD>\n};
                    print qq{        <TD ALIGN="left" VALIGN="middle"><TEXTAREA ROWS="6" COLS="80" WRAP="physical">$name</TEXTAREA></TD>\n};
                    $flag = 1;
                }
                print qq{    </TR>\n};
            }
        }
    }
    print "</TABLE>\n";
}
else
{
    print "<H3>There are currently no registered special events besides the regular trainings.</H3>\n";
}

print <<"VERBATIM";
    </CENTER>
    </TD></TR></TABLE>
    <P>
    <HR NOSHADE SIZE="2">
    <P>
</CENTER>
</BODY>
</HTML>
VERBATIM

sub process_query_string
{
    my $query = $ENV{'QUERY_STRING'} || $ENV{'REDIRECT_QUERY_STRING'} || join('&', @ARGV) || '';
    my @pairs = split(/&/, $query);
    my($pair,$var,$val);

    foreach $pair (@pairs)
    {
        ($var,$val) = split(/=/,$pair,2);
        if ($var =~ /^[a-z]+$/i)
        {
            $val =~ s/\+/ /g;
            $val =~ s/%([0-9A-Fa-f][0-9A-Fa-f])/chr(hex($1))/eg;
            $val =~ s/^\s+//;
            $val =~ s/\s+$//;
            if ($var =~ /^l(?:ang(?:uage)?)?$/i)
            {
                if (($val =~ /^\d+$/ or ($val = Decode_Language($val))) and $val >= 1 and $val <= 6) { $Lang = $val; }
            }
        }
    }
}

__END__

